import { Box, Paper, Link, useMediaQuery, Portal, List, ListItem, ListItemIcon, Tooltip, ListItemText } from '@material-ui/core';
import {useTheme, makeStyles, Theme} from '@material-ui/core/styles';
import React, { useEffect, useState } from 'react';
import ContentAccessor,{QueryListOption, Document} from '../accessor/document';
import {Link as RouterLink} from 'react-router-dom';
import { LoadingCircle, ContentInfo, NavList, NavItem } from './mod';
import {toQueryString} from '../accessor/util';


const useStyles = makeStyles((theme:Theme)=>({
    root:{
        display:"grid",
        gridGap: theme.spacing(4),
    },
    anchor_thumbnail:{
        background: '#272733',
        display: 'flex',
        alignItems: 'center',
        justifyContent: 'center',
        [theme.breakpoints.up("sm")]:{
            width: theme.spacing(25),
            height: theme.spacing(25),
            flexShrink: 0,
        }
    },
    contentPaper:{
        overflowY:'hidden',
        display:'flex',
        flexDirection: 'column',
        [theme.breakpoints.up("sm")]:{
            height:200,
            flexDirection: 'row',
        },
    },
    content_thumnail: {
        maxWidth: '100%',
        maxHeight: '100%',
    },
}));

export type GalleryProp = {
    option?:QueryListOption;
};
type GalleryState = {
    documents:Document[]|undefined;
}

export const GalleryInfo = (props: GalleryProp)=>{
    const [state,setState]= useState<GalleryState>({documents:undefined});
    useEffect(()=>{
        const load = (async ()=>{
            const c = await ContentAccessor.findList(props.option);
            //todo : if c is undefined, retry to fetch 3 times. and show error message.
            setState({documents:c});
        })
        load();
    },[props.option]);
    const classes = useStyles();
    const queryString = toQueryString(props.option||{});

    if(state.documents === undefined){
        return (<LoadingCircle/>);
    }
    else{
        return (<Box className={classes.root}>{
            state.documents.map(x=>{
                return (<ContentInfo document={x} key={x.id} 
                    gallery={`/search?${queryString}`} short/>);
            })
            }
            </Box>);
    }
}