import React from 'react';
import {ChipProps} from '@material-ui/core/Chip';
import { Chip, colors } from '@material-ui/core';
import {useTheme, makeStyles, Theme, emphasize, fade} from '@material-ui/core/styles';

type TagChipStyleProp = {
    color: string
}

const useTagStyles = makeStyles((theme:Theme)=>({
    root:(props:TagChipStyleProp)=>({
        color: theme.palette.getContrastText(props.color),
        backgroundColor: props.color,
    }),
    clickable:(props:TagChipStyleProp)=>({
        '&:hover, &:focus':{
            backgroundColor:emphasize(props.color,0.08)
        }
    }),
    deletable: {
        '&:focus': {
            backgroundColor: (props:TagChipStyleProp)=>emphasize(props.color, 0.2),
        }
    },
    outlined:{
        color: (props:TagChipStyleProp)=>props.color,
        border: (props:TagChipStyleProp)=> `1px solid ${props.color}`,
        '$clickable&:hover, $clickable&:focus, $deletable&:focus': {
            backgroundColor:(props:TagChipStyleProp)=>fade(props.color,theme.palette.action.hoverOpacity),
        },
    },
    icon:{
        color:"inherit",
    },
    deleteIcon:{
        color:(props:TagChipStyleProp)=>fade(theme.palette.getContrastText(props.color),0.7),
        "&:hover, &:active":{
            color:(props:TagChipStyleProp)=>theme.palette.getContrastText(props.color),
        }
    }
}));

const {blue, pink} = colors;
const getTagColorName = (tagname :string):string=>{
    if(tagname.startsWith("female")){
        return pink['700'];
    }
    else if(tagname.startsWith("male")){
        return blue[600];
    }
    else return "default";
}
export const ColorChip = (props:Omit<ChipProps,"color"> & {color: string})=>{
    const {color,...rest} = props;
    const classes = useTagStyles({color : color !== "default" ? color : "#000"});
    return <Chip color="default" classes={{
        ...(color !== "default" ? classes : {})
    }} {...rest}></Chip>;
}

export const TagChip = (props:Omit<ChipProps,"color"> & {tagname: string})=>{
    const {tagname,...rest} = props;
    return <ColorChip color={getTagColorName(tagname)} {...rest}></ColorChip>;
}