import React, { createContext, useRef, useState } from 'react';

export const UserContext = createContext({
    username: "",
    permission: [] as string[],
    setUsername: (s: string) => { },
    setPermission: (permission: string[]) => { }
});

type LoginLocalStorage = {
    username: string,
    permission: string[],
    refreshExpired: number
};

let localObj: LoginLocalStorage|null = null;

export const getInitialValue = async () => {
    if(localObj === null){
        const storagestr = window.localStorage.getItem("UserLoginContext") as string | null;
        const storage = storagestr !== null ? JSON.parse(storagestr) as LoginLocalStorage | null : null;
        localObj = storage;
    }
    if (localObj !== null && localObj.refreshExpired > Math.floor(Date.now() / 1000)) {
        return {
            username: localObj.username,
            permission: localObj.permission,
        }
    }
    const res = await fetch('/user/refresh', {
        method: 'POST',
    });
    if (res.status !== 200) throw new Error("Maybe Network Error")
    const r = await res.json() as LoginLocalStorage & { refresh: boolean };
    if (r.refresh) {
        localObj = {
            username: r.username,
            permission: r.permission,
            refreshExpired: r.refreshExpired
        }
        window.localStorage.setItem("UserLoginContext", JSON.stringify(localObj));
    }
    return {
        username: r.username,
        permission: r.permission
    }
}
export const doLogout = async ()=>{
    const res = await fetch('/user/logout', {
        method: 'POST',
    });
    await res.json();
    localObj.refreshExpired = 0;
    localObj.username = "";
    localObj.permission = [];
    window.localStorage.setItem("UserLoginContext", JSON.stringify(localObj));
}