import Knex from 'knex';
import {IUser,UserCreateInput, UserAccessor, Password} from '../model/user';

type PermissionTable = {
    username:string,
    name:string
};
type DBUser = {
    username : string,
    password_hash: string,
    password_salt: string
}
class KnexUser implements IUser{
    private knex: Knex;
    readonly username: string;
    readonly password: Password;
    
    constructor(username: string, pw: Password,knex:Knex){
        this.username = username;
        this.password = pw;
        this.knex = knex;
    }
    async reset_password(password: string){
        this.password.set_password(password);
        await this.knex.from("users")
            .where({username:this.username})
            .update({password_hash:this.password.hash,password_salt:this.password.salt});
    }
    async get_permissions(){
        let b = (await this.knex.select('*').from("permissions")
            .where({username : this.username})) as PermissionTable[];
        return b.map(x=>x.name);
    }
    async add(name: string) {
        if(!(await this.get_permissions()).includes(name)){
            const r = await this.knex.insert({
                username: this.username,
                name: name
            }).into("permissions");
            return true;
        }
        return false;
    }
    async remove(name: string) {
        const r = await this.knex
            .from("permissions")
            .where({
                username:this.username, name:name
            }).delete();
        return r !== 0;
    }
}

export const createKnexUserController = (knex: Knex):UserAccessor=>{
    const createUserKnex = async (input:UserCreateInput)=>{
        if(undefined !== (await findUserKenx(input.username))){
            return undefined;
        }
        const user = new KnexUser(input.username,new Password(input.password),knex);
        await knex.insert<DBUser>({
            username: user.username,
            password_hash: user.password.hash,
            password_salt: user.password.salt}).into("users");
        return user;
    };
    const findUserKenx = async (id:string)=>{
        let user:DBUser[] = await knex.select("*").from("users").where({username:id});
        if(user.length == 0) return undefined;
        const first = user[0];
        return new KnexUser(first.username,
            new Password({hash: first.password_hash, salt: first.password_salt}), knex);
    }
    const delUserKnex = async (id:string) => {
        let r = await knex.delete().from("users").where({username:id});
        return r===0;
    }
    return {
        createUser: createUserKnex,
        findUser: findUserKenx,
        delUser: delUserKnex,
    };
}