import {join as pathjoin} from 'path';
import {Document, DocumentAccessor} from '../model/mod';
import { ContentFile, createContentFile } from '../content/mod';
import {IDiffWatcher} from './watcher';
import {ContentList} from './content_list';

//refactoring needed.
export class ContentDiffHandler{
    waiting_list:ContentList;
    tombstone: Map<string,Document>;//hash, contentfile
    doc_cntr: DocumentAccessor;
    content_type: string;
    constructor(cntr: DocumentAccessor,content_type:string){
        this.waiting_list = new ContentList();
        this.tombstone = new Map<string,Document>();
        this.doc_cntr = cntr;
        this.content_type = content_type;
    }
    async setup(){
        const deleted = await this.doc_cntr.findDeleted(this.content_type);
        for (const it of deleted) {
            this.tombstone.set(it.content_hash,it);
        }
    }
    register(diff:IDiffWatcher){
        diff.on('create',(filename)=>this.OnCreated(diff.path,filename))
            .on('delete',(filename)=>this.OnDeleted(diff.path,filename))
            .on('change',(prev_filename,cur_filename)=>this.OnChanged(diff.path,prev_filename,cur_filename));
    }
    private async OnDeleted(basepath:string,filename:string){
        const cpath = pathjoin(basepath,filename);
        if(this.waiting_list.hasPath(cpath)){
            this.waiting_list.deletePath(cpath);
            return;
        }
        const dbc = await this.doc_cntr.findByPath(basepath,filename);
        if(dbc.length === 0) return; //ignore
        if(this.waiting_list.hasHash(dbc[0].content_hash)){
            //if path changed, update changed path.
            await this.doc_cntr.update({
                id:dbc[0].id,
                deleted_at: null,
                filename:filename,
                basepath:basepath
            });
            return;
        }
        //db invalidate
        await this.doc_cntr.update({
            id:dbc[0].id,
            deleted_at: Date.now(),
        });
        this.tombstone.set(dbc[0].content_hash, dbc[0]);
    }
    private async OnCreated(basepath:string,filename:string){
        const content = createContentFile(this.content_type,pathjoin(basepath,filename));
        const hash = await content.getHash();
        const c = this.tombstone.get(hash);
        if(c !== undefined){
            this.doc_cntr.update({
                id: c.id,
                deleted_at: null,
                filename:filename,
                basepath:basepath
            });
            return;
        }
        this.waiting_list.set(content);
    }
    private async OnChanged(basepath:string,prev_filename:string,cur_filename:string){
        const doc = await this.doc_cntr.findByPath(basepath,prev_filename);
        await this.doc_cntr.update({...doc[0],filename:cur_filename});
    }
}