import { DefaultContext, Middleware, Next, ParameterizedContext } from 'koa';
import compose from 'koa-compose';
import Router, { IParamMiddleware } from 'koa-router';
import { ContentContext } from './context';
import MangaRouter from './manga';
import VideoRouter from './video';

const table:{[s:string]:Router|undefined} = {
    "manga": new MangaRouter,
    "video": new VideoRouter
}
const all_middleware = (cont: string|undefined, restarg: string|undefined)=>async (ctx:ParameterizedContext<ContentContext,DefaultContext>,next:Next)=>{
    if(cont == undefined){
        ctx.status = 404;
        return;
    }
    if(ctx.state.location.type != cont){
        console.error("not matched")
        ctx.status = 404;
        return;
    }
    const router = table[cont];
    if(router == undefined){
        ctx.status = 404;
        return;
    }
    const rest = "/"+(restarg || "");
    const result = router.match(rest,"GET");
    if(!result.route){
        return await next();
    }
    const chain = result.pathAndMethod.reduce((combination : Middleware<any& DefaultContext,any>[],cur)=>{
        combination.push(async (ctx,next)=>{
            const captures = cur.captures(rest);
            ctx.params = cur.params(rest,captures);
            ctx.request.params = ctx.params;
            ctx.routerPath = cur.path;
            return await next();
        });
        return combination.concat(cur.stack);
    },[]);
    return await compose(chain)(ctx,next);
};
export class AllContentRouter extends Router<ContentContext>{
    constructor(){
        super();
        this.get('/:content_type',async (ctx,next)=>{
            return await (all_middleware(ctx.params["content_type"],undefined))(ctx,next);
        });
        this.get('/:content_type/:rest(.*)', async (ctx,next) => {
            const cont = ctx.params["content_type"] as string;
            return await (all_middleware(cont,ctx.params["rest"]))(ctx,next);
        });
    }
};